@@ -1,9 +1,10 @@
-#' `Tidy up' R code while preserving comments
-#' 
-#' This function has nothing to do with code optimization; it just returns
-#' parsed source code, but also tries to preserve comments, which is different
-#' with \code{\link[base]{parse}}. See `Details'.
-#' 
+#' Reformat R code while preserving blank lines and comments
+#'
+#' This function returns reformatted source code; it tries to preserve blank
+#' lines and comments, which is different with \code{\link[base]{parse}}. It can
+#' also replace \code{=} with \code{<-} where \code{=} means assignments, and
+#' reindent code by a specified number of spaces (default is 4).
+#'
 #' This function helps the users to tidy up their source code in a sense that
 #' necessary indents and spaces will be added, but comments will be preserved if
 #' \code{keep.comment = TRUE}. See the references to know how this function
@@ -17,6 +18,8 @@
 #'   of comments (default \code{FALSE})
 #' @param replace.assign whether to replace the assign operator \code{=} with
 #'   \code{<-}
+#' @param left.brace.newline whether to put the left brace \code{\{} to a new
+#'   line (default \code{FALSE})
 #' @param reindent.spaces number of spaces to indent the code (default 4)
 #' @param output output to the console or a file using \code{\link[base]{cat}}?
 #' @param text an alternative way to specify the input: if it is \code{NULL},
@@ -33,18 +36,17 @@
 #' @return A list with components \item{text.tidy}{the reformatted code as a
 #'   character vector} \item{text.mask}{the code containing comments, which are
 #'   masked in assignments or with the weird operator}
-#'   \item{begin.comment,end.comment}{identifiers used to mark the comments}
 #' @note Be sure to read the reference to know other limitations.
 #' @author Yihui Xie <\url{http://yihui.name}> with substantial contribution
 #'   from Yixuan Qiu <\url{http://yixuan.cos.name}>
-#' @seealso \code{\link[base]{parse}}, \code{\link[base]{deparse}}, 
+#' @seealso \code{\link[base]{parse}}, \code{\link[base]{deparse}},
 #'   \code{\link[base]{cat}}
 #' @references \url{https://github.com/yihui/formatR/wiki/} (an introduction to
 #'   this package, with examples and further notes)
-#'   
-#'   The package vignette also contains some examples (see 
+#'
+#'   The package vignette also contains some examples (see
 #'   \code{vignette('formatR', package = 'formatR')}.
-#'   
+#'
 #'   Hadley's style guide: \url{https://github.com/hadley/devtools/wiki/Style}
 #' @keywords IO
 #' @export
@@ -53,6 +55,7 @@ tidy.source = function(source = "clipboard", keep.comment = getOption('keep.comm
                        keep.blank.line = getOption('keep.blank.line', TRUE),
                        keep.space = getOption('keep.space', FALSE),
                        replace.assign = getOption('replace.assign', FALSE),
+                       left.brace.newline = getOption('left.brace.newline', FALSE),
                        reindent.spaces = getOption('reindent.spaces', 4),
                        output = TRUE, text = NULL,
                        width.cutoff = getOption("width"), ...) {
@@ -62,30 +65,27 @@ tidy.source = function(source = "clipboard", keep.comment = getOption('keep.comm
     }
     text = readLines(source, warn = FALSE)
   }
-  text.lines = text
-  if (identical(text.lines, '')) {
+  if (length(text) == 0L || all(grepl('^\\s*$', text))) {
     if (output) cat('\n', ...)
-    return('')
+    return(list(text.tidy = '', text.mask = ''))
   }
-  if (isTRUE(keep.comment)) {
-    ## if you have variable names like this in your code, then you really beat me...
-    begin.comment = ".BeGiN_TiDy_IdEnTiFiEr_HaHaHa"
-    end.comment = ".HaHaHa_EnD_TiDy_IdEnTiFiEr"
+  text.lines = text
+  if (keep.comment) {
     if (!keep.space) text.lines = gsub("^[[:space:]]+|[[:space:]]+$", "", text.lines)
     head.comment = grepl('^[[:space:]]*#', text.lines)
     if (any(head.comment)) {
       text.lines[head.comment] = gsub('"', "'", text.lines[head.comment])
-      text.lines[head.comment] = gsub("\\", "\\\\", text.lines[head.comment], fixed = TRUE)
     }
     ## wrap long comments if you do not want to preserve leading spaces
     if (!keep.space) {
-      text.lines = reflow.comments(text.lines, head.comment, width.cutoff)
+      head.comment = head.comment & !grepl("^\\s*#+'", text.lines)
+      text.lines = reflow_comments(text.lines, head.comment, width.cutoff)
       head.comment = grepl('^[[:space:]]*#', text.lines)
     }
     text.lines[head.comment] =
       sprintf('invisible("%s%s%s")', begin.comment, text.lines[head.comment], end.comment)
     blank.line = grepl('^[[:space:]]*$', text.lines)
-    if (any(blank.line) && isTRUE(keep.blank.line)) {
+    if (any(blank.line) && keep.blank.line) {
       ## no blank lines before an 'else' statement!
       else.line = grep('^[[:space:]]*else(\\W|)', text.lines)
       for (i in else.line) {
@@ -98,30 +98,26 @@ tidy.source = function(source = "clipboard", keep.comment = getOption('keep.comm
       }
       text.lines[blank.line] = sprintf('invisible("%s%s")', begin.comment, end.comment)
     }
-    text.lines = mask.inline(text.lines, replace.assign, begin.comment, end.comment)
-    text.mask = tidy.block(text.lines, width.cutoff)
-    text.tidy = unmask.source(text.mask)
-  } else {
-    text.mask = tidy.block(text.lines, width.cutoff)
-    text.tidy = unlist(strsplit(text.mask, '\n', fixed = TRUE))
-    begin.comment = end.comment = ""
+    text.lines = mask_inline(text.lines)
   }
-  text.tidy = reindent.lines(text.tidy, reindent.spaces)
+  text.mask = tidy_block(text.lines, width.cutoff, replace.assign)
+  text.tidy = if (keep.comment) unmask.source(text.mask) else text.mask
+  text.tidy = reindent_lines(text.tidy, reindent.spaces)
+  if (left.brace.newline) text.tidy = move_leftbrace(text.tidy, reindent.spaces)
   if (output) cat(paste(text.tidy, collapse = "\n"), "\n", ...)
-  invisible(list(text.tidy = text.tidy, text.mask = text.mask,
-                 begin.comment = begin.comment, end.comment = end.comment))
+  invisible(list(text.tidy = text.tidy, text.mask = text.mask))
 }
 
+## if you have variable names like this in your code, then you really beat me...
+begin.comment = ".BeGiN_TiDy_IdEnTiFiEr_HaHaHa"
+end.comment = ".HaHaHa_EnD_TiDy_IdEnTiFiEr"
+pat.comment = paste('invisible\\("\\', begin.comment, '|\\', end.comment, '"\\)', sep = '')
+
 # wrapper around parse() and deparse()
-tidy.block = function(text, width) {
-  exprs = base::parse(text = text)
-  n = length(exprs)
-  res = character(n)
-  for (i in 1:n) {
-    dep = paste(base::deparse(exprs[i], width), collapse = "\n")
-    res[i] = substring(dep, 12, nchar(dep) - 1)
-  }
-  res
+tidy_block = function(text, width = getOption('width'), arrow = FALSE) {
+  exprs = base::parse(text = text, srcfile = NULL)
+  exprs = if (arrow) replace_assignment(exprs) else as.list(exprs)
+  sapply(exprs, function(e) paste(base::deparse(e, width), collapse = '\n'))
 }
 
 #' Restore the real source code from the masked text
@@ -154,17 +150,13 @@ unmask.source = function(text.mask) {
   text.mask = gsub("%InLiNe_IdEnTiFiEr%[ ]*\n", "%InLiNe_IdEnTiFiEr%", text.mask)
   ## move 'else ...' back to the last line
   text.mask = gsub('\n[[:space:]]*else', ' else', text.mask)
-  text.mask = unlist(strsplit(text.mask, '\n', fixed = TRUE))
-  idx = grepl('invisible(".BeGiN_TiDy_IdEnTiFiEr_HaHaHa', text.mask, fixed = TRUE)
-  text.mask[idx] = gsub('\\\\', '\\', text.mask[idx], fixed = TRUE)
-  text.tidy = gsub('invisible\\("\\.BeGiN_TiDy_IdEnTiFiEr_HaHaHa|\\.HaHaHa_EnD_TiDy_IdEnTiFiEr"\\)', 
-                   '', text.mask)
-  text.tidy = gsub(' %InLiNe_IdEnTiFiEr%[ ]*"([ ]*#[^"]*)"', "  \\1", text.tidy)
+  text.tidy = gsub(pat.comment, '', text.mask)
+  gsub(' %InLiNe_IdEnTiFiEr%[ ]*"([ ]*#[^"]*)"', "  \\1", text.tidy)
 }
 
 
 #' A weird operator for internal use only
-#' 
+#'
 #' This operator is almost meaningless; it is used to mask the inline comments.
 #' @rdname InLiNe_IdEnTiFiEr
 #' @usage x %InLiNe_IdEnTiFiEr% y
@@ -178,31 +170,31 @@ unmask.source = function(text.mask) {
 #' @keywords internal
 #' @examples
 #' ## we can use it with *anything*
-#' 
+#'
 #' 1 %InLiNe_IdEnTiFiEr% 2
-#' 
+#'
 #' 1:10 %InLiNe_IdEnTiFiEr% "asdf"
-#' 
+#'
 #' lm %InLiNe_IdEnTiFiEr% 'garbage'
-#' 
+#'
 "%InLiNe_IdEnTiFiEr%" <- function(x, y) x
 
 
 #' Modified versions of parse() and deparse()
-#' 
+#'
 #' These two functions parse and deparse the masked source code.
-#' 
+#'
 #' For \code{\link{parse.tidy}}, the source code is masked to preserve comments,
 #' then this function uses \code{\link[base]{parse}} to return the parsed but
 #' unevaluated expressions in a list.
-#' 
+#'
 #' For \code{\link{deparse.tidy}}, it uses \code{\link[base]{deparse}} to turn
 #' the unevaluated (and masked) expressions into character strings; the masks
 #' will be removed to restore the real source code. See
 #' \code{\link{unmask.source}}.
-#' @param text the source code as a character string to be passed to 
+#' @param text the source code as a character string to be passed to
 #'   \code{\link{tidy.source}}
-#' @param ... for \code{\link{parse.tidy}}: other arguments to be passed to 
+#' @param ... for \code{\link{parse.tidy}}: other arguments to be passed to
 #'   \code{\link{tidy.source}}; for \code{\link{deparse.tidy}}: arguments to be
 #'   passed to \code{\link[base]{deparse}}
 #' @return \code{\link{parse.tidy}} returns the unevaluated expressions;
@@ -210,7 +202,7 @@ unmask.source = function(text.mask) {
 #' @author Yihui Xie <\url{http://yihui.name}>
 #' @note These functions are mainly designed for the package \pkg{pgfSweave};
 #'   they may not be useful to general users.
-#' @seealso \code{\link[base]{parse}}, \code{\link[base]{deparse}}, 
+#' @seealso \code{\link[base]{parse}}, \code{\link[base]{deparse}},
 #'   \code{\link{tidy.source}}
 #' @export
 #' @keywords internal
@@ -223,13 +215,13 @@ unmask.source = function(text.mask) {
 #' "2+2+2    # 'short comments'",
 #' "lm(y~x1+x2)  ### only 'single quotes' are allowed in comments",
 #' "1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1  ## comments after a long line")
-#' 
+#'
 #' (expr = parse.tidy(src))
-#' 
+#'
 #' parse.tidy(src, keep.blank.line = TRUE)
-#' 
+#'
 #' cat(deparse.tidy(expr))
-#' 
+#'
 #' deparse.tidy(expr, width.cutoff = 50)
 parse.tidy = function(text, ...) {
   tidy.res = tidy.source(text = text, output = FALSE, ...)
@@ -245,12 +237,12 @@ deparse.tidy = function(expr, ...) {
 }
 
 #' Format the R scripts under a directory
-#' 
+#'
 #' This function first looks for all the R scripts under a directory (using the
 #' pattern \code{"\\\\.[RrSsQq]$"}), then uses \code{\link{tidy.source}} to tidy
 #' these scripts. The original scripts will be overwritten with reformatted code
 #' if reformatting was successful. You may need to back up the original
-#' directory first if you do not fully understand the tricks 
+#' directory first if you do not fully understand the tricks
 #' \code{\link{tidy.source}} is using.
 #' @param path the directory
 #' @param recursive whether to recursively look for R scripts under \code{path}
@@ -261,7 +253,7 @@ deparse.tidy = function(expr, ...) {
 #' @export
 #' @examples
 #' library(formatR)
-#' 
+#'
 #' path = tempdir()
 #' file.copy(system.file('demo', package = 'base'), path, recursive=TRUE)
 #' tidy.dir(path, recursive=TRUE)
